{***************************************************************}
{              XJustOne Component for Delphi 32                 }
{ Version:     2.0                                              }
{ Author:      Gennady Makeev (makeev@cacr.ioc.ac.ru)           }
{ Improvement: Aleksey Xacker (xacker@phreaker.net)             }
{ Home Page:   http://www.angen.com/~xacker/                    }
{***************************************************************}
{ This component let start only one copy of your application.   }
{***************************************************************}
{ PROPERTIES:                                                   }
{   Terminate: Boolean - Terminate second copy                  }
{ EVENTS:                                                       }
{   AlreadyExist - Run if application already started           }
{***************************************************************}

unit JustOne;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms;

type
  TXJustOne = class(TComponent)
  private
    FTerminate: Boolean;
    FAlreadyExist: TNotifyEvent;
    Mutex: hWnd;
    IsCreated: Boolean;
  public
    procedure Loaded; override;
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Terminate: Boolean read FTerminate write FTerminate;
    property AlreadyExist: TNotifyEvent read FAlreadyExist write FAlreadyExist;
  end;

procedure Register;

implementation

procedure TXJustOne.Loaded;
begin
  inherited Loaded;
  IsCreated := False;
  Mutex := CreateMutex(nil, False, PChar(ExtractFileName(Application.ExeName)));
  if (GetLastError = ERROR_ALREADY_EXISTS) or
     (Mutex = 0) then
  begin
   if Assigned(FAlreadyExist) then FAlreadyExist(Self);
   if FTerminate then
    Application.Terminate;
   Exit;
  end;
  IsCreated := True;
end;

constructor TXJustOne.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FTerminate := True;
end;

destructor TXJustOne.Destroy;
begin
  if IsCreated then
   CloseHandle(Mutex);
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Xacker', [TXJustOne]);
end;

end.
